{% load i18n %}
{% load humanize %}
{% load static %}
<table class="table table-striped table-hover w-100" id="planets-details">
    <thead>
        <th class="col-character w-auto">{% translate "Character" %}</th>
        <th class="col-planet-type w-auto">{% translate "Planet" %}</th>
        <th class="col-upgrade-level w-auto">{% translate "Upgrade Level" %}</th>
        <th class="col-products w-auto no-sort">{% translate "Products" %}</th>
        <th class="col-extractor w-auto">{% translate "Extractor" %}</th>
        <th class="col-Status w-auto no-sort">{% translate "Status" %}</th>
        <th class="col-Last-Updated w-auto">{% translate "Last Updated" %}</th>
        <th class="col-Actions w-auto no-sort">{% translate "Actions" %}</th>
    </thead>

    <tbody>
    </tbody>
</table>

<!-- Confirmation Modal -->
<div class="modal fade" id="confirmModal" tabindex="-1" role="dialog" aria-labelledby="confirmModalLabel" aria-hidden="true">
    <div class="modal-dialog modal-dialog-centered" role="document">
        <div class="modal-content">
            <div class="modal-header">
                <h5 class="modal-title" id="confirmModalLabel">{% translate "Confirm Action" %}</h5>
                <button type="button" class="btn-close py-3 px-3" data-bs-dismiss="modal" aria-label="Close"></button>
            </div>
            <div class="modal-body">
                <span id="confirm-request">
                <!-- Confirmation message will be populated here -->
                </span>
            </div>
            <div class="modal-footer">
                <button type="button" class="btn btn-secondary" data-bs-dismiss="modal">{% translate "Cancel" %}</button>
                <button type="button" class="btn btn-primary" id="finalizeActionButton">{% translate "Confirm" %}</button>
            </div>
        </div>
    </div>
</div>
