noData = [
    {
        "ratting": [],
        "total": {
            "total_amount": "0.00",
            "total_amount_ess": "0.00",
            "total_amount_all": "0.00",
            "total_amount_mining": "0.00",
            "total_amount_others": "0.00",
            "total_amount_costs": "0.00",
        },
    }
]

CharmonthlyMarch = [
    {
        "ratting": [
            {
                "main_id": 1001,
                "main_name": "Gneuten",
                "entity_type": "character",
                "total_amount": "200000.00",
                "total_amount_ess": "1133333.33",
                "total_amount_mining": "200000",
                "total_amount_others": "200000.00",
                "total_amount_costs": "-400000.00",
            }
        ],
        "total": {
            "total_amount": "200000.00",
            "total_amount_ess": "1133333.33",
            "total_amount_all": "1333333.33",
            "total_amount_mining": "200000.00",
            "total_amount_others": "200000.00",
            "total_amount_costs": "-400000.00",
        },
    }
]

CharmonthlyMarchWithTaxEvent = [
    {
        "ratting": [
            {
                "main_id": 1001,
                "main_name": "Gneuten",
                "entity_type": "character",
                "total_amount": "200000.00",
                "total_amount_ess": "0.00",
                "total_amount_mining": "200000",
                "total_amount_others": "200000.00",
                "total_amount_costs": "-400000.00",
            }
        ],
        "total": {
            "total_amount": "200000.00",
            "total_amount_ess": "0.00",
            "total_amount_all": "200000.00",
            "total_amount_mining": "200000.00",
            "total_amount_others": "200000.00",
            "total_amount_costs": "-400000.00",
        },
    }
]

CharmonthlyMarchMulti = [
    {
        "ratting": [
            {
                "main_id": 1001,
                "main_name": "Gneuten",
                "entity_type": "character",
                "total_amount": "200000.00",
                "total_amount_ess": "1133333.33",
                "total_amount_mining": "200000",
                "total_amount_others": "200000.00",
                "total_amount_costs": "-400000.00",
            },
            {
                "main_id": 1002,
                "main_name": "rotze Rotineque",
                "entity_type": "character",
                "total_amount": "0.00",
                "total_amount_ess": "0.00",
                "total_amount_mining": "100000",
                "total_amount_others": "0.00",
                "total_amount_costs": "0.00",
            },
        ],
        "total": {
            "total_amount": "200000.00",
            "total_amount_ess": "1133333.33",
            "total_amount_all": "1433333.33",
            "total_amount_mining": "300000.00",
            "total_amount_others": "200000.00",
            "total_amount_costs": "-400000.00",
        },
    }
]

Charyearly = [
    {
        "ratting": [
            {
                "main_id": 1001,
                "main_name": "Gneuten",
                "entity_type": "character",
                "total_amount": "300000.00",
                "total_amount_ess": "1133333.33",
                "total_amount_mining": "200000",
                "total_amount_others": "200000.00",
                "total_amount_costs": "-400000.00",
            }
        ],
        "total": {
            "total_amount": "300000.00",
            "total_amount_ess": "1133333.33",
            "total_amount_all": "1433333.33",
            "total_amount_mining": "200000.00",
            "total_amount_others": "200000.00",
            "total_amount_costs": "-400000.00",
        },
    }
]

CharNoMining = [
    {
        "ratting": [
            {
                "main_id": 1001,
                "main_name": "Gneuten",
                "entity_type": "character",
                "total_amount": "200000.00",
                "total_amount_ess": "1133333.33",
                "total_amount_mining": "0.00",
                "total_amount_others": "200000.00",
                "total_amount_costs": "-400000.00",
            }
        ],
        "total": {
            "total_amount": "200000.00",
            "total_amount_ess": "1133333.33",
            "total_amount_all": "1133333.33",
            "total_amount_mining": "0.00",
            "total_amount_others": "200000.00",
            "total_amount_costs": "-400000.00",
        },
    }
]
