{% load i18n %}
{% load humanize %}
{% load static %}
<table class="table table-striped table-hover w-100 d-none" id="ratting">
    <thead>
        <th class="col-main-character w-auto">{% trans "Main Character" %}</th>
        <th class="col-total-amount w-auto">{% trans "Ratting Amount" %}</th>
        <th class="col-total-ess w-auto">{% trans "ESS Payout" %}</th>
        {% if entity_type == 'character' %}
            <th class="col-total-mining w-auto">{% trans "Mining Amount" %}</th>
        {% endif %}
        <th class="col-total-others w-auto">{% trans "Misc. Amount" %}</th>
        {% if entity_type == 'character' %}
            <th class="col-total-singlecosts w-auto">{% trans "Costs" %}</th>
        {% endif %}
        <th class="col-total-action w-auto"></th>
    </thead>

    <tbody>
    </tbody>

    <tfoot id="foot" style="display: none;">
        <tr>
            <th><strong>{% trans "Summary" %}</strong></th>
            <th class="col-total-amount"></th>
            <th class="col-total-ess"></th>
            {% if entity_type == 'character' %}
                <th class="col-total-mining"></th>
            {% endif %}
            <th class="col-total-others"></th>
            {% if entity_type == 'character' %}
                <th class="col-total-costs"></th>
            {% endif %}
            <th></th>
        </tr>
        <tr>
            <th><strong>{% trans "Total" %}</strong></th>
            {% if entity_type == 'character' %}
                <th></th>
                <th></th>
            {% endif %}
            <th></th>
            <th></th>
            <th class="col-total-gesamt"></th>
            <th class="col-total-button"></th>
        </tr>
    </tfoot>
</table>
