noData = [
    {
        "billboard": {
            "standard": {
                "charts": None,
                "rattingbar": None,
                "workflowgauge": None,
            },
        },
    }
]

CharmonthlyMarch = [
    {
        "billboard": {
            "standard": {
                "charts": {
                    "title": "Donut Chart",
                    "date": "2025-01-31",
                    "categories": [
                        "bounty",
                        "contract",
                        "donation",
                        "ess",
                        "insurance",
                        "market",
                        "mining",
                        "production",
                    ],
                    "series": [
                        {
                            "date": "2025-01-31",
                            "BOUNTY (income)": {"value": 200000.00, "mode": "income"},
                            "CONTRACT (income)": {"value": 100000.00, "mode": "income"},
                            "DONATION (income)": {"value": 100000.00, "mode": "income"},
                            "ESS (income)": {"value": 1133333.33, "mode": "income"},
                            "INSURANCE (cost)": {"value": 100000.00, "mode": "cost"},
                            "MARKET (cost)": {"value": 200000.00, "mode": "cost"},
                            "MINING (income)": {"value": 200000.00, "mode": "income"},
                            "PRODUCTION (cost)": {"value": 100000.00, "mode": "cost"},
                        }
                    ],
                },
                "rattingbar": {
                    "title": "Ratting Chart",
                    "date": "2025-01-31",
                    "categories": ["Bounty", "ESS", "Miscellaneous", "Mining"],
                    "series": [
                        {"date": "2024-03-16", "mining": 200000},
                        {
                            "date": "2024-03-19",
                            "bounty": 200000,
                            "ess": 1133333,
                            "miscellaneous": 200000,
                        },
                    ],
                },
                "workflowgauge": {
                    "title": "Workflow Chart",
                    "date": "2025-01-31",
                    "categories": ["bounty", "ess", "mining", "miscellaneous"],
                    "series": [
                        {
                            "date": "2025-01-31",
                            "BOUNTY": {"value": 200000.00, "mode": "income"},
                            "ESS": {"value": 1133333.33, "mode": "income"},
                            "MINING": {"value": 200000.00, "mode": "income"},
                            "MISCELLANEOUS": {"value": 200000.00, "mode": "income"},
                        }
                    ],
                },
            }
        }
    }
]


CharmonthlyMarchMulti = [
    {
        "billboard": {
            "standard": {
                "charts": {
                    "title": "Donut Chart",
                    "date": "2025-01-31",
                    "categories": [
                        "bounty",
                        "contract",
                        "donation",
                        "ess",
                        "insurance",
                        "market",
                        "mining",
                        "production",
                    ],
                    "series": [
                        {
                            "date": "2025-01-31",
                            "BOUNTY (income)": {"value": 200000.00, "mode": "income"},
                            "CONTRACT (income)": {"value": 100000.00, "mode": "income"},
                            "DONATION (income)": {"value": 100000.00, "mode": "income"},
                            "ESS (income)": {"value": 1133333.33, "mode": "income"},
                            "INSURANCE (cost)": {"value": 100000.00, "mode": "cost"},
                            "MARKET (cost)": {"value": 200000.00, "mode": "cost"},
                            "MINING (income)": {"value": 300000.00, "mode": "income"},
                            "PRODUCTION (cost)": {"value": 100000.00, "mode": "cost"},
                        }
                    ],
                },
                "rattingbar": {
                    "title": "Ratting Chart",
                    "date": "2025-01-31",
                    "categories": ["Bounty", "ESS", "Miscellaneous", "Mining"],
                    "series": [
                        {"date": "2024-03-16", "mining": 300000},
                        {
                            "date": "2024-03-19",
                            "bounty": 200000,
                            "ess": 1133333,
                            "miscellaneous": 200000,
                        },
                    ],
                },
                "workflowgauge": {
                    "title": "Workflow Chart",
                    "date": "2025-01-31",
                    "categories": ["bounty", "ess", "mining", "miscellaneous"],
                    "series": [
                        {
                            "date": "2025-01-31",
                            "BOUNTY": {"value": 200000.00, "mode": "income"},
                            "ESS": {"value": 1133333.33, "mode": "income"},
                            "MINING": {"value": 300000.00, "mode": "income"},
                            "MISCELLANEOUS": {"value": 200000.00, "mode": "income"},
                        }
                    ],
                },
            }
        }
    }
]

Charyearly = [
    {
        "billboard": {
            "standard": {
                "charts": {
                    "title": "Donut Chart",
                    "date": "2025-01-31",
                    "categories": [
                        "bounty",
                        "contract",
                        "donation",
                        "ess",
                        "insurance",
                        "market",
                        "mining",
                        "production",
                    ],
                    "series": [
                        {
                            "date": "2025-01-31",
                            "BOUNTY (income)": {"value": 300000.00, "mode": "income"},
                            "CONTRACT (income)": {"value": 100000.00, "mode": "income"},
                            "DONATION (income)": {"value": 100000.00, "mode": "income"},
                            "ESS (income)": {"value": 1133333.33, "mode": "income"},
                            "INSURANCE (cost)": {"value": 100000.00, "mode": "cost"},
                            "MARKET (cost)": {"value": 200000.00, "mode": "cost"},
                            "MINING (income)": {"value": 200000.00, "mode": "income"},
                            "PRODUCTION (cost)": {"value": 100000.00, "mode": "cost"},
                        }
                    ],
                },
                "rattingbar": {
                    "title": "Ratting Chart",
                    "date": "2025-01-31",
                    "categories": ["Bounty", "ESS", "Miscellaneous", "Mining"],
                    "series": [
                        {"date": "2024-01", "bounty": 100000},
                        {
                            "date": "2024-03",
                            "bounty": 200000,
                            "ess": 1133333,
                            "miscellaneous": 200000,
                            "mining": 200000,
                        },
                    ],
                },
                "workflowgauge": {
                    "title": "Workflow Chart",
                    "date": "2025-01-31",
                    "categories": [
                        "bounty",
                        "ess",
                        "mining",
                        "miscellaneous",
                    ],
                    "series": [
                        {
                            "date": "2025-01-31",
                            "BOUNTY": {"value": 300000.00, "mode": "income"},
                            "ESS": {"value": 1133333.33, "mode": "income"},
                            "MINING": {"value": 200000.00, "mode": "income"},
                            "MISCELLANEOUS": {"value": 200000.00, "mode": "income"},
                        }
                    ],
                },
            }
        }
    }
]

CharNoMining = [
    {
        "billboard": {
            "standard": {
                "charts": {
                    "title": "Donut Chart",
                    "date": "2025-01-31",
                    "categories": [
                        "bounty",
                        "contract",
                        "donation",
                        "ess",
                        "insurance",
                        "market",
                        "production",
                    ],
                    "series": [
                        {
                            "date": "2025-01-31",
                            "BOUNTY (income)": {"value": 200000.00, "mode": "income"},
                            "CONTRACT (income)": {"value": 100000.00, "mode": "income"},
                            "DONATION (income)": {"value": 100000.00, "mode": "income"},
                            "ESS (income)": {"value": 1133333.33, "mode": "income"},
                            "INSURANCE (cost)": {"value": 100000.00, "mode": "cost"},
                            "MARKET (cost)": {"value": 200000.00, "mode": "cost"},
                            "PRODUCTION (cost)": {"value": 100000.00, "mode": "cost"},
                        }
                    ],
                },
                "rattingbar": {
                    "title": "Ratting Chart",
                    "date": "2025-01-31",
                    "categories": ["Bounty", "ESS", "Miscellaneous", "Mining"],
                    "series": [
                        {
                            "date": "2024-03-19",
                            "bounty": 200000,
                            "ess": 1133333,
                            "miscellaneous": 200000,
                        }
                    ],
                },
                "workflowgauge": {
                    "title": "Workflow Chart",
                    "date": "2025-01-31",
                    "categories": ["bounty", "ess", "mining", "miscellaneous"],
                    "series": [
                        {
                            "date": "2025-01-31",
                            "BOUNTY": {"value": 200000.00, "mode": "income"},
                            "ESS": {"value": 1133333.33, "mode": "income"},
                            "MISCELLANEOUS": {"value": 200000.00, "mode": "income"},
                        }
                    ],
                },
            }
        }
    }
]
