planet_many = [
    {
        "character_id": 1001,
        "character_name": "Gneuten",
        "planet": "Test Planet I",
        "planet_id": 4001,
        "planet_type_id": 13,
        "upgrade_level": 5,
        "expiry_date": "2024-08-26T17:17:02Z",
        "expired": True,
        "alarm": False,
        "extractors": {
            "1046238237375": {
                "install_time": "2024-08-12T17:17:02Z",
                "expiry_time": "2024-08-26T17:17:02Z",
                "item_id": 2268,
                "item_name": "Aqueous Liquids",
                "progress_percentage": 100.0,
            },
            "1046238237378": {
                "install_time": "2024-08-12T17:17:02Z",
                "expiry_time": "2024-08-26T17:17:02Z",
                "item_id": 2309,
                "item_name": "Ionic Solutions",
                "progress_percentage": 100.0,
            },
        },
        "products": {
            "raw": {
                "2268": {
                    "id": 2268,
                    "name": "Aqueous Liquids",
                    "category": "Planet Liquid-Gas - Raw Resource",
                },
                "2309": {
                    "id": 2309,
                    "name": "Ionic Solutions",
                    "category": "Planet Liquid-Gas - Raw Resource",
                },
            },
            "processed": {
                "9832": {
                    "id": 9832,
                    "name": "Coolant",
                    "category": "Refined Commodities - Tier 2",
                },
                "3645": {
                    "id": 3645,
                    "name": "Water",
                    "category": "Basic Commodities - Tier 1",
                },
                "2390": {"id": 2390, "name": "Electrolytes", "category": "Planet"},
            },
        },
        "storage": {
            "1036002868084": {
                "facility_id": 2557,
                "facility_name": "Storm Launchpad",
                "contents": [
                    {"amount": 14800, "type_id": 2390, "product_name": "Electrolytes"}
                ],
            },
            "1046238237381": {
                "facility_id": 2543,
                "facility_name": "Gas Launchpad",
                "contents": [],
            },
            "1046238237382": {
                "facility_id": 2543,
                "facility_name": "Gas Launchpad",
                "contents": [],
            },
        },
        "facility": None,
        "last_update": None,
    },
    {
        "character_id": 1001,
        "character_name": "Gneuten",
        "planet": "Test Planet I",
        "planet_id": 4002,
        "planet_type_id": 13,
        "upgrade_level": 5,
        "expiry_date": "2024-08-26T17:17:02Z",
        "expired": True,
        "alarm": False,
        "extractors": {
            "1046238237375": {
                "install_time": "2024-08-12T17:17:02Z",
                "expiry_time": "2024-08-26T17:17:02Z",
                "item_id": 2268,
                "item_name": "Aqueous Liquids",
                "progress_percentage": 100.0,
            },
            "1046238237378": {
                "install_time": "2024-08-12T17:17:02Z",
                "expiry_time": "2024-08-26T17:17:02Z",
                "item_id": 2309,
                "item_name": "Ionic Solutions",
                "progress_percentage": 100.0,
            },
        },
        "products": {
            "raw": {
                "2268": {
                    "id": 2268,
                    "name": "Aqueous Liquids",
                    "category": "Planet Liquid-Gas - Raw Resource",
                },
                "2309": {
                    "id": 2309,
                    "name": "Ionic Solutions",
                    "category": "Planet Liquid-Gas - Raw Resource",
                },
            },
            "processed": {
                "9832": {
                    "id": 9832,
                    "name": "Coolant",
                    "category": "Refined Commodities - Tier 2",
                },
                "3645": {
                    "id": 3645,
                    "name": "Water",
                    "category": "Basic Commodities - Tier 1",
                },
                "2390": {"id": 2390, "name": "Electrolytes", "category": "Planet"},
            },
        },
        "storage": {
            "1036002868084": {
                "facility_id": 2557,
                "facility_name": "Storm Launchpad",
                "contents": [
                    {"amount": 14800, "type_id": 2390, "product_name": "Electrolytes"}
                ],
            },
            "1046238237381": {
                "facility_id": 2543,
                "facility_name": "Gas Launchpad",
                "contents": [],
            },
            "1046238237382": {
                "facility_id": 2543,
                "facility_name": "Gas Launchpad",
                "contents": [],
            },
        },
        "facility": None,
        "last_update": None,
    },
]

planet_single = [
    {
        "character_id": 1001,
        "character_name": "Gneuten",
        "planet": "Test Planet I",
        "planet_id": 4001,
        "planet_type_id": 13,
        "upgrade_level": 5,
        "expiry_date": "2024-08-26T17:17:02Z",
        "expired": False,
        "alarm": False,
        "extractors": {
            "1046238237375": {
                "install_time": "2024-08-12T17:17:02Z",
                "expiry_time": "2024-08-26T17:17:02Z",
                "item_id": 2268,
                "item_name": "Aqueous Liquids",
                "progress_percentage": 57.14,
            },
            "1046238237378": {
                "install_time": "2024-08-12T17:17:02Z",
                "expiry_time": "2024-08-26T17:17:02Z",
                "item_id": 2309,
                "item_name": "Ionic Solutions",
                "progress_percentage": 57.14,
            },
        },
        "products": {
            "raw": {
                "2268": {
                    "id": 2268,
                    "name": "Aqueous Liquids",
                    "category": "Planet Liquid-Gas - Raw Resource",
                },
                "2309": {
                    "id": 2309,
                    "name": "Ionic Solutions",
                    "category": "Planet Liquid-Gas - Raw Resource",
                },
            },
            "processed": {
                "9832": {
                    "id": 9832,
                    "name": "Coolant",
                    "category": "Refined Commodities - Tier 2",
                },
                "3645": {
                    "id": 3645,
                    "name": "Water",
                    "category": "Basic Commodities - Tier 1",
                },
                "2390": {"id": 2390, "name": "Electrolytes", "category": "Planet"},
            },
        },
        "storage": {
            "1036002868084": {
                "facility_id": 2557,
                "facility_name": "Storm Launchpad",
                "contents": [
                    {"amount": 14800, "type_id": 2390, "product_name": "Electrolytes"}
                ],
            },
            "1046238237381": {
                "facility_id": 2543,
                "facility_name": "Gas Launchpad",
                "contents": [],
            },
            "1046238237382": {
                "facility_id": 2543,
                "facility_name": "Gas Launchpad",
                "contents": [],
            },
        },
        "facility": None,
        "last_update": None,
    }
]
